@@ -9,11 +9,11 @@ RDF::RDFa::Parser::Trine - Use a RDF::Trine::Model for the returned RDF graph
 
 =head1 VERSION
 
-Version 0.11
+Version 0.2
 
 =cut
 
-our $VERSION = 0.11;
+our $VERSION = 0.2;
 
 use RDF::RDFa::Parser;
 use RDF::Trine::Model;
@@ -23,7 +23,6 @@ sub new {
   my $class = shift;
   my $store = shift;
   my $self  = $class->SUPER::new(@_);
-  $self->{NAMED_GRAPH} = undef;
   $self->{RESULTS} = RDF::Trine::Model->new($store);
   $self->set_callbacks(\&_callback_resource, \&_callback_literal);
   bless ($self, $class);
@@ -51,57 +50,102 @@ the graph method, and will instead return an RDF::Trine::Model.
 The constructor. Has three parameters. The first is a (single)
 RDF::Trine::Store. The others are passed directly to the superclass.
 
-=item $p->graph
+=item $p->graph( [ $graph_name ] ) 
 
-Will return an RDF::Trine::Model object containing the full graph. As
-per the RDFa specification, it will always return an unnamed graph
-containing all the triples of the RDFa document.
+Without a graph name, this method will return an RDF::Trine::Model
+object with all statements of the full graph. As per the RDFa
+specification, it will always return an unnamed graph containing all
+the triples of the RDFa document. If the model contains multiple
+graphs, all triples will be returned unless a graph name is specified.
+
+It will also take an optional graph URI as argument, and return an
+RDF::Trine::Model tied to a temporary storage with all triples in that
+graph.
+
+
+
+=cut
+
+sub graph {
+  my $self = shift;
+  my $graph = shift;
+  if (defined($graph)) {
+    my $tg;
+    if ($graph =~ m/^_:(.*)/) {
+      $tg = RDF::Trine::Node::Blank->new($1);
+    } else {
+      $tg = RDF::Trine::Node::Resource->new($graph, $self->{baseuri});
+    }
+    my $storage = RDF::Trine::Store::DBI->temporary_store;
+    my $m = RDF::Trine::Model->new($storage);
+    my $i = $self->{RESULTS}->get_statements(undef, undef, undef, $tg);
+    while (my $statement = $i->next) {
+      $m->add_statement($statement);
+    }
+    return $m;
+  } else {
+    return $self->{RESULTS};
+  }
+}
+
+
+=item $p->graphs
+
+Will return a hashref of all named graphs, where the graph name is a
+key and the value is a RDF::Trine::Model tied to a temporary storage.
 
 =back
 
 =cut
 
 
-sub named_graph { #TODO
-  my ($self, $switch) = @_;
- # $self->{NAMED_GRAPH} = $switch;
- $self->{NAMED_GRAPH} = 0;
-}
-
-sub graph {
+sub graphs {
   my $self = shift;
-  return $self->{RESULTS};
+  my @graphs = keys(%{$self->SUPER::graphs});
+  my %result;
+  foreach my $graph (@graphs) {
+    $result{$graph} = $self->graph($graph);
+  }
+  return \%result;
 }
 
-
 sub _callback {
-  my $parser      = shift;  # A reference to the RDF::RDFa::Parser object
+  my $parser    = shift;  # A reference to the RDF::RDFa::Parser object
   my $element   = shift;  # A reference to the XML::LibXML element being parsed
   my $subject   = shift;  # Subject URI or bnode
   my $predicate = shift;  # Predicate URI
   my $to        = shift;  # RDF::Trine::Node Resource URI or bnode
-
+  my $graph     = shift;  # Graph URI or bnode (if named graphs feature is enabled)
+  
   # First, make sure subject and predicates are the right kind of nodes
   my $tp = RDF::Trine::Node::Resource->new($predicate, $parser->{baseuri});
   my $ts;
-  if ($subject =~ m/^_:/) {
-    $ts = RDF::Trine::Node::Blank->new($subject);
+  if ($subject =~ m/^_:(.*)/) {
+    $ts = RDF::Trine::Node::Blank->new($1);
   } else {
     $ts = RDF::Trine::Node::Resource->new($subject, $parser->{baseuri});
   }
 
-  # Add statement to default model.
-  my $statement = RDF::Trine::Statement->new($ts, $tp, $to);
-
-
-  $parser->{RESULTS}->add_statement($statement);
   # If we are configured for it, and graph name can be found, add it.
-  if ($parser->{NAMED_GRAPH}) {
-    my $graph_name = _find_graph_name($element);
-    if ($graph_name) {
-      my $statement = RDF::Trine::Statement::Quad->new($ts, $tp, $to, $graph_name);
-      $parser->{RESULTS}->add_statement($statement);
+  if (ref($parser->{named_graphs}) && ($graph)) {
+    my $tg;
+    if ($graph =~ m/^_:(.*)/) {
+      $tg = RDF::Trine::Node::Blank->new($1);
+    } else {
+      $tg = RDF::Trine::Node::Resource->new($graph, $parser->{baseuri});
+    }   
+    my $statement = RDF::Trine::Statement::Quad->new($ts, $tp, $to, $tg);
+    $parser->{RESULTS}->add_statement($statement);
+    if ($graph ne 'RDFaDefaultGraph') {
+      my $graph_statement = RDF::Trine::Statement::Quad->new($ts, $tp, $to, 
+				   RDF::Trine::Node::Blank->new('RDFaDefaultGraph'));
+      $parser->{RESULTS}->add_statement($graph_statement, 
+				   RDF::Trine::Node::Blank->new('RDFaDefaultGraph'));
     }
+  } else {
+    # If no graph name, just add triples
+    my $statement = RDF::Trine::Statement->new($ts, $tp, $to);
+    $parser->{RESULTS}->add_statement($statement);
   }
 }
 
@@ -113,17 +157,18 @@ sub _callback_resource {
   my $subject   = shift;  # Subject URI or bnode
   my $predicate = shift;  # Predicate URI
   my $object    = shift;  # Resource URI or bnode
+  my $graph     = shift;  # Graph URI or bnode (if named graphs feature is enabled)
 
   # First make sure the object node type is ok.
   my $to;
-  if ($object =~ m/^_:/) {
-    $to = RDF::Trine::Node::Blank->new($object);
+  if ($object =~ m/^_:(.*)/) {
+    $to = RDF::Trine::Node::Blank->new($1);
   } else {
     $to = RDF::Trine::Node::Resource->new($object, $parser->{baseuri});
   }
 
   # Run the common function
-  _callback($parser, $element, $subject, $predicate, $to);
+  _callback($parser, $element, $subject, $predicate, $to, $graph);
 
 }
 
@@ -135,6 +180,7 @@ sub _callback_literal {
   my $object    = shift;  # Resource Literal
   my $datatype  = shift;  # Datatype URI (possibly undef or '')
   my $language  = shift;  # Language (possibly undef or '')
+  my $graph     = shift;  # Graph URI or bnode (if named graphs feature is enabled)
 
   # Now we know there's a literal
 
@@ -151,21 +197,15 @@ sub _callback_literal {
     $to = RDF::Trine::Node::Literal->new($object, $language, undef);
   }
 
-  _callback($parser, $element, $subject, $predicate, $to);
+  _callback($parser, $element, $subject, $predicate, $to, $graph);
 }
 
-sub _find_graph_name {
-  return 0;
-}
 
 
 =head1 AUTHOR
 
 Kjetil Kjernsmo, C<< <kjetilk at cpan.org> >>
 
-=head1 TODO
-
-This module does not yet fully support the named graphs of its subclass.
 
 =head1 BUGS
 
